Package: grpreg
Title: Regularization Paths for Regression Models with Grouped
        Covariates
Version: 3.6.0
Authors@R: c(
  person("Patrick", "Breheny", role=c("aut","cre"), email="patrick-breheny@uiowa.edu", comment=c(ORCID="0000-0002-0650-1119")),
  person("Yaohui", "Zeng", role="ctb"),
  person("Ryan", "Miller", role="aut", comment=c(ORCID="0000-0003-0446-9992")),
  person("Ryan", "Kurth", role="ctb"))
Description: Efficient algorithms for fitting the regularization path of linear
  regression, GLM, and Cox regression models with grouped penalties.  This
  includes group selection methods such as group lasso, group MCP, and
  group SCAD as well as bi-level selection methods such as the group
  exponential lasso, the composite MCP, and the group bridge.  For more
  information, see Breheny and Huang (2009) <doi:10.4310/sii.2009.v2.n3.a10>,
  Huang, Breheny, and Ma (2012) <doi:10.1214/12-sts392>, Breheny and Huang
  (2015) <doi:10.1007/s11222-013-9424-2>, and Breheny (2015)
  <doi:10.1111/biom.12300>, or visit the package homepage
  <https://pbreheny.github.io/grpreg/>.
License: GPL-3
URL: https://pbreheny.github.io/grpreg/,
        https://github.com/pbreheny/grpreg
BugReports: https://github.com/pbreheny/grpreg/issues
Depends: R (>= 3.1.0)
Imports: Matrix
Suggests: knitr, rmarkdown, splines, survival, tinytest
VignetteBuilder: knitr
Encoding: UTF-8
Config/roxygen2/version: 7.3.3.9000
NeedsCompilation: yes
Packaged: 2026-03-25 20:23:03 UTC; pbreheny
Author: Patrick Breheny [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-0650-1119>),
  Yaohui Zeng [ctb],
  Ryan Miller [aut] (ORCID: <https://orcid.org/0000-0003-0446-9992>),
  Ryan Kurth [ctb]
Maintainer: Patrick Breheny <patrick-breheny@uiowa.edu>
Repository: CRAN
Date/Publication: 2026-03-26 13:10:07 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 23:54:14 UTC; unix
Archs: grpreg.so.dSYM
